#include <stdio.h>
#include "kk_tsl_api.h"
#include "kk_dm_mng.h"
#include "sqlite3.h"
#include "kk_log.h"
#include "kk_scene_handle.h"
#include "cJSON.h"

extern sqlite3 *g_kk_pDb;

typedef struct {
    void *mutex;
    sqlite3 *pDb;	
} kk_scene_ctx_t;

typedef enum{
	DB_IDX = 0,
	DB_ROOM_NAME,
	DB_ROOM_ID,		
};

typedef enum{
	DB_DEV_IDX = 0,
	DB_DEV_ROOM_ID,
	DB_DEV_DEVICECODE,	
	DB_DEV_EPNUM,
	
};

static kk_scene_ctx_t s_kk_scene_ctx = {NULL,NULL};

static kk_scene_ctx_t *_kk_scene_get_ctx(void)
{
    return &s_kk_scene_ctx;
}
static void _kk_scene_lock(void)
{
    kk_scene_ctx_t *ctx = _kk_scene_get_ctx();
    if (ctx->mutex) {
        HAL_MutexLock(ctx->mutex);
    }
}

static void _kk_scene_unlock(void)
{
    kk_scene_ctx_t *ctx = _kk_scene_get_ctx();
	
    if (ctx->mutex) {
        HAL_MutexUnlock(ctx->mutex);
    }
}

static int _kk_scene_db_init(void)
{
    kk_scene_ctx_t *ctx = _kk_scene_get_ctx();
    char *pcErr;

    //eUtils_LockLock(&sLock);
    _kk_scene_lock();
    ctx->pDb = g_kk_pDb;
   
    INFO_PRINT("scene db Database opened\n");
    
	const char *pSceneTable = "CREATE TABLE IF NOT EXISTS SceneInfo( \
		name varchar(255), \
		sceneType INTEGER, \
		enable INTEGER, \
		sceneId varchar(255) UNIQUE)";

    if (sqlite3_exec(ctx->pDb, pSceneTable, NULL, NULL, &pcErr) != SQLITE_OK)
    {
        ERROR_PRINT("Error creating table (%s)\n", pcErr);
        sqlite3_free(pcErr);
        //eUtils_LockUnlock(&sLock);
        _kk_scene_unlock();
        return FAIL_RETURN;
    }

	const char *pSceneTriggerTable = "CREATE TABLE IF NOT EXISTS SceneTriggerInfo( \
		type varchar(255), \
		deviceCode varchar(255), \
		epNum INTEGER, \
		propertyName varchar(255), \
		compareType varchar(255), \
		compareValue varchar(255), \
		sceneId varchar(255))";


    if (sqlite3_exec(ctx->pDb, pSceneTriggerTable, NULL, NULL, &pcErr) != SQLITE_OK)
    {
        ERROR_PRINT("Error creating table (%s)\n", pcErr);
        sqlite3_free(pcErr);
        //eUtils_LockUnlock(&sLock);
        _kk_scene_unlock();
        return FAIL_RETURN;
    }

	const char *pSceneConditionTable = "CREATE TABLE IF NOT EXISTS SceneConditionInfo( \
		type varchar(255), \
		startTime INTEGER, \
		endTime INTEGER, \
		crossDay INTEGER, \
		repeat_days INTEGER, \
		sceneId varchar(255))";


    if (sqlite3_exec(ctx->pDb, pSceneConditionTable, NULL, NULL, &pcErr) != SQLITE_OK)
    {
        ERROR_PRINT("Error creating table (%s)\n", pcErr);
        sqlite3_free(pcErr);
        //eUtils_LockUnlock(&sLock);
        _kk_scene_unlock();
        return FAIL_RETURN;
    }	

	const char *pSceneActionTable = "CREATE TABLE IF NOT EXISTS SceneActionInfo( \
		type varchar(255), \
		deviceCode varchar(255), \
		epNum INTEGER, \
		propertyName varchar(255), \
		propertyValue varchar(255), \
		delay INTEGER, \
		sceneId varchar(255))";


    if (sqlite3_exec(ctx->pDb, pSceneActionTable, NULL, NULL, &pcErr) != SQLITE_OK)
    {
        ERROR_PRINT("Error creating table (%s)\n", pcErr);
        sqlite3_free(pcErr);
        //eUtils_LockUnlock(&sLock);
        _kk_scene_unlock();
        return FAIL_RETURN;
    }	

    _kk_scene_unlock();
    return SUCCESS_RETURN;
}

#define KK_DEVICE_TSL_TYPE (0x08)
static kk_tsl_t *s_scene_shadow = NULL;
static int kk_scene_tsl_load(void)
{
	int res = 0;
	char *tsl_str = NULL;
	int heartTime = 0;
	tsl_str = kk_load_json("15", KK_DEVICE_TSL_TYPE);
	if(tsl_str != NULL)
	{
		printf("[%s][%d]\n",__FUNCTION__,__LINE__);
		res = kk_tsl_create(tsl_str,strlen(tsl_str),&s_scene_shadow,&heartTime);
		free(tsl_str);
		if(res != 0){
			return FAIL_RETURN;
		}
	}
	printf("[%s][%d]\n",__FUNCTION__,__LINE__);
	return SUCCESS_RETURN;

}
kk_tsl_t * kk_scene_shadow(void)
{
	return s_scene_shadow;
}

int kk_scene_init(void)
{
	int res = 0;
    kk_scene_ctx_t *ctx = _kk_scene_get_ctx();

	printf("[%s][%d]\n",__FUNCTION__,__LINE__);

    /* Create Mutex */
    ctx->mutex = HAL_MutexCreate();
    if (ctx->mutex == NULL) {
        return FAIL_RETURN;
    }
	_kk_scene_db_init();

	res = kk_scene_tsl_load();
	if(res != SUCCESS_RETURN){
		ERROR_PRINT("[%s][%d]kk_scene_init FAIL!!!\n",__FUNCTION__,__LINE__);

	}
	printf("[%s][%d]\n",__FUNCTION__,__LINE__);
	return SUCCESS_RETURN;
}

#if 0
uint64_t kk_room_add(const char *name)
{
	int res = 0;
    kk_area_ctx_t *ctx = _kk_area_get_ctx();
	char *sqlCmd = NULL;
	char roomId[32] = {0};
	char *zErrMsg = 0;
	uint64_t u64RoomId = 0;

	const char *insertCmd = "insert into AreaInfo (name, roomId) \
								values ('%s','%s');";						
	_kk_area_lock();
	ctx->roomNum++;
	u64RoomId = get_unique_id();
	sprintf(roomId,"%u",u64RoomId);

	sqlCmd = sqlite3_mprintf(insertCmd,name,roomId);	
	res = sqlite3_exec(ctx->pDb, sqlCmd, NULL, NULL, &zErrMsg);
	if( res != SQLITE_OK ){
	   ERROR_PRINT("SQL error: %s\n", zErrMsg);
	   sqlite3_free(zErrMsg);
	}else{
	   //INFO_PRINT("sub device insert data successfully\n");
	}
	sqlite3_free(sqlCmd);
	_kk_area_unlock();
	return u64RoomId;
}

int kk_room_delete(const char *roomId)
{
	int res = 0;
    kk_area_ctx_t *ctx = _kk_area_get_ctx();
	char *sqlCmd = NULL;
	char *zErrMsg = 0;
	const char *deleteCmd = "delete from AreaInfo where roomId = '%s';";

								
	_kk_area_lock();
	 sqlCmd = sqlite3_mprintf(deleteCmd,roomId);	

	INFO_PRINT("Table delete data sqlCmd:%s\n",sqlCmd);
	res = sqlite3_exec(ctx->pDb, sqlCmd, NULL, NULL, &zErrMsg);
	if( res != SQLITE_OK ){
	   ERROR_PRINT("SQL error: %s\n", zErrMsg);
	   sqlite3_free(zErrMsg);
	}else{
	   //INFO_PRINT("Table delete data successfully\n");
	   ctx->roomNum--;
	}
	sqlite3_free(sqlCmd);
	
	_kk_area_unlock();
	return SUCCESS_RETURN;
}

int kk_room_dev_add(const char *roomId,const char *deviceCode,const char *epNum)
{
	int res = 0;
    kk_area_ctx_t *ctx = _kk_area_get_ctx();
	char *sqlCmd = NULL;
	char *zErrMsg = 0;

	const char *insertCmd = "insert into AreaDevInfo (roomId, deviceCode, epNum) \
								values ('%s','%s','%s');";	
								
	_kk_area_lock();
	sqlCmd = sqlite3_mprintf(insertCmd,roomId,deviceCode,epNum);	
	res = sqlite3_exec(ctx->pDb, sqlCmd, NULL, NULL, &zErrMsg);
	if( res != SQLITE_OK ){
	   ERROR_PRINT("SQL error: %s\n", zErrMsg);
	   sqlite3_free(zErrMsg);
	}else{
	   //INFO_PRINT("sub device insert data successfully\n");
	}
	sqlite3_free(sqlCmd);
	_kk_area_unlock();
	return SUCCESS_RETURN;
}

int kk_room_dev_remove(const char *deviceCode,const char *epNum)
{
	int res = 0;
    kk_area_ctx_t *ctx = _kk_area_get_ctx();
	char *sqlCmd = NULL;
	char *zErrMsg = 0;

	const char *insertCmd = "delete from AreaDevInfo where deviceCode = '%s' and epNum = '%s';";	
								
	_kk_area_lock();
	sqlCmd = sqlite3_mprintf(insertCmd,deviceCode,epNum);	
	res = sqlite3_exec(ctx->pDb, sqlCmd, NULL, NULL, &zErrMsg);
	if( res != SQLITE_OK ){
	   ERROR_PRINT("SQL error: %s\n", zErrMsg);
	   sqlite3_free(zErrMsg);
	}else{
	   //INFO_PRINT("sub device insert data successfully\n");
	}
	sqlite3_free(sqlCmd);
	_kk_area_unlock();
	return SUCCESS_RETURN;
}



int kk_room_device_list_add(const char *deviceCode)
{
	int len;
	kk_dev_list_t *ptr,*pre;
	
	ptr = pre = s_device_list;
	if(deviceCode == NULL){
		return FAIL_RETURN;
	}
	
	while(ptr!=NULL){
		pre = ptr;
		if(strcmp(ptr->deviceCode,deviceCode) == 0){
			return SUCCESS_RETURN;
		}
		ptr = ptr->next;
	}
	ptr = (kk_dev_list_t*)malloc(sizeof(kk_dev_list_t));
	memset(ptr,0,sizeof(kk_dev_list_t));
	memcpy(ptr->deviceCode,deviceCode,strlen(deviceCode));

	if(s_device_list==NULL){
		s_device_list = ptr;
	}else{
		pre->next = ptr;
	}

	return SUCCESS_RETURN;
}

kk_dev_list_t*  kk_get_room_deviceCode(const char* roomId)
{
	sqlite3_stmt *stmt;
	char *sqlCmd = NULL;
    kk_area_ctx_t *ctx = _kk_area_get_ctx();
	const char *searchCmd = "select * from AreaDevInfo;";	
	printf("");
	_kk_area_lock();
	sqlCmd = sqlite3_mprintf(searchCmd,roomId);	
	sqlite3_prepare_v2(ctx->pDb, sqlCmd, strlen(sqlCmd), &stmt, NULL);
	//INFO_PRINT("total_column = %d\n", sqlite3_column_count(stmt));
	while(sqlite3_step(stmt) == SQLITE_ROW){
	   if(strcmp(sqlite3_column_text(stmt, DB_DEV_ROOM_ID),roomId) == 0){
	   		kk_room_device_list_add(sqlite3_column_text(stmt, DB_DEV_DEVICECODE));
	   }
    }
	sqlite3_finalize(stmt);
	_kk_area_unlock();
	return s_device_list;
}
void kk_free_room_dev_list(void)
{
	kk_dev_list_t *ptr = s_device_list;
	kk_dev_list_t *ptmp = NULL;
	while(ptr != NULL){
		ptmp = ptr->next;
		free(ptr);
		ptr = NULL;
		ptr = ptmp;
	}
	s_device_list = NULL;
}
#endif
static int kk_scene_insert_scene_info(const char* name,const char* sceneType,const char* enable,const char* sceneId)
{
	int res = 0;
    kk_scene_ctx_t *ctx = _kk_scene_get_ctx();
	char *sqlCmd = NULL;
	char *zErrMsg = 0;

	const char *insertCmd = "insert into SceneInfo (name, sceneType,enable,sceneId) \
								values ('%s','%d','%d','%s');";	
								
	_kk_scene_lock();
	sqlCmd = sqlite3_mprintf(insertCmd,name,sceneType,enable,sceneId);	
	res = sqlite3_exec(ctx->pDb, sqlCmd, NULL, NULL, &zErrMsg);
	if( res != SQLITE_OK ){
	   ERROR_PRINT("SQL error: %s\n", zErrMsg);
	   sqlite3_free(zErrMsg);
	   sqlite3_free(sqlCmd);
	   _kk_scene_unlock();	
	   return FAIL_RETURN;
	}
	
	sqlite3_free(sqlCmd);
	_kk_scene_unlock();
	return SUCCESS_RETURN;

}
static int kk_scene_insert_scene_trigger(const char* type,const char* deviceCode,const char* epNum,const char* propertyName,
												const char* compareType,const char* compareValue,const char* sceneId)
{
	int res = 0;
    kk_scene_ctx_t *ctx = _kk_scene_get_ctx();
	char *sqlCmd = NULL;
	char *zErrMsg = 0;

	const char *insertCmd = "insert into SceneTriggerInfo (type, deviceCode,epNum,propertyName,compareType,compareValue,sceneId) \
								values ('%s','%s','%d','%s','%s','%s','%s');";	
								
	_kk_scene_lock();
	sqlCmd = sqlite3_mprintf(insertCmd,type,deviceCode,epNum,propertyName,compareType,compareValue,sceneId);	
	res = sqlite3_exec(ctx->pDb, sqlCmd, NULL, NULL, &zErrMsg);
	if( res != SQLITE_OK ){
	   ERROR_PRINT("SQL error: %s\n", zErrMsg);
	   sqlite3_free(zErrMsg);
	   sqlite3_free(sqlCmd);
	   _kk_scene_unlock();	
	   return FAIL_RETURN;	
	}
	sqlite3_free(sqlCmd);
	_kk_scene_unlock();
	
	return SUCCESS_RETURN;

}

static int kk_scene_insert_scene_condition(const char* type,int startTime,int endTime,int crossDay,
												char repeat_days,const char* sceneId)
{
	int res = 0;
	kk_scene_ctx_t *ctx = _kk_scene_get_ctx();
	char *sqlCmd = NULL;
	char *zErrMsg = 0;

	const char *insertCmd = "insert into SceneConditionInfo (type, startTime,endTime,crossDay,repeat_days,sceneId) \
								values ('%s','%d','%d','%d','%d','%s');";	
								
	_kk_scene_lock();
	sqlCmd = sqlite3_mprintf(insertCmd,type,startTime,endTime,crossDay,repeat_days,sceneId);	
	res = sqlite3_exec(ctx->pDb, sqlCmd, NULL, NULL, &zErrMsg);
	if( res != SQLITE_OK ){
	   ERROR_PRINT("SQL error: %s\n", zErrMsg);
	   sqlite3_free(zErrMsg);
	   sqlite3_free(sqlCmd);
	   _kk_scene_unlock();	
	   return FAIL_RETURN;	
	}
	sqlite3_free(sqlCmd);
	_kk_scene_unlock();
	
	return SUCCESS_RETURN;

}


static int kk_scene_insert_scene_action(const char* type,const char* deviceCode,int epNum,const char* propertyName,
												const char* propertyValue,int delay,const char* sceneId)
{
	int res = 0;
	kk_scene_ctx_t *ctx = _kk_scene_get_ctx();
	char *sqlCmd = NULL;
	char *zErrMsg = 0;

	const char *insertCmd = "insert into SceneActionInfo (type, deviceCode,epNum,propertyName,propertyValue,delay,sceneId) \
								values ('%s','%s','%d','%s','%s','%d','%s');";	
								
	_kk_scene_lock();
	sqlCmd = sqlite3_mprintf(insertCmd,type,deviceCode,epNum,propertyName,propertyValue,delay,sceneId);	
	res = sqlite3_exec(ctx->pDb, sqlCmd, NULL, NULL, &zErrMsg);
	if( res != SQLITE_OK ){
	   ERROR_PRINT("SQL error: %s\n", zErrMsg);
	   sqlite3_free(zErrMsg);
	   sqlite3_free(sqlCmd);
	   _kk_scene_unlock();	
	   return FAIL_RETURN;	
	}
	sqlite3_free(sqlCmd);
	_kk_scene_unlock();
	
	return SUCCESS_RETURN;

}
int kk_scene_delete_scene_info(const char *sceneId)
{
	int res = 0;
	kk_scene_ctx_t *ctx = _kk_scene_get_ctx();
	char *sqlCmd = NULL;
	char *zErrMsg = 0;
	const char *deleteCmd = "delete from SceneInfo where sceneId = '%s';";
						
	_kk_scene_lock();
	 sqlCmd = sqlite3_mprintf(deleteCmd,sceneId);	
	res = sqlite3_exec(ctx->pDb, sqlCmd, NULL, NULL, &zErrMsg);
	if( res != SQLITE_OK ){
	   ERROR_PRINT("SQL error: %s\n", zErrMsg);
	   sqlite3_free(zErrMsg);
	   sqlite3_free(sqlCmd);
	   _kk_scene_unlock();	
	   return FAIL_RETURN;		
	}
	sqlite3_free(sqlCmd);
	_kk_scene_unlock();
	return SUCCESS_RETURN;
}
int kk_scene_delete_scene_trigger(const char *sceneId)
{
	int res = 0;
	kk_scene_ctx_t *ctx = _kk_scene_get_ctx();
	char *sqlCmd = NULL;
	char *zErrMsg = 0;
	const char *deleteCmd = "delete from SceneTriggerInfo where sceneId = '%s';";
						
	_kk_scene_lock();
	 sqlCmd = sqlite3_mprintf(deleteCmd,sceneId);	
	res = sqlite3_exec(ctx->pDb, sqlCmd, NULL, NULL, &zErrMsg);
	if( res != SQLITE_OK ){
	   ERROR_PRINT("SQL error: %s\n", zErrMsg);
	   sqlite3_free(zErrMsg);
	   sqlite3_free(sqlCmd);
	   _kk_scene_unlock();	
	   return FAIL_RETURN;		
	}
	sqlite3_free(sqlCmd);
	_kk_scene_unlock();
	return SUCCESS_RETURN;
}
int kk_scene_delete_scene_condition(const char *sceneId)
{
	int res = 0;
	kk_scene_ctx_t *ctx = _kk_scene_get_ctx();
	char *sqlCmd = NULL;
	char *zErrMsg = 0;
	const char *deleteCmd = "delete from SceneConditionInfo where sceneId = '%s';";
						
	_kk_scene_lock();
	 sqlCmd = sqlite3_mprintf(deleteCmd,sceneId);	
	res = sqlite3_exec(ctx->pDb, sqlCmd, NULL, NULL, &zErrMsg);
	if( res != SQLITE_OK ){
	   ERROR_PRINT("SQL error: %s\n", zErrMsg);
	   sqlite3_free(zErrMsg);
	   sqlite3_free(sqlCmd);
	   _kk_scene_unlock();	
	   return FAIL_RETURN;		
	}
	sqlite3_free(sqlCmd);
	_kk_scene_unlock();
	return SUCCESS_RETURN;
}

int kk_scene_delete_scene_action(const char *sceneId)
{
	int res = 0;
	kk_scene_ctx_t *ctx = _kk_scene_get_ctx();
	char *sqlCmd = NULL;
	char *zErrMsg = 0;
	const char *deleteCmd = "delete from SceneActionInfo where sceneId = '%s';";
						
	_kk_scene_lock();
	 sqlCmd = sqlite3_mprintf(deleteCmd,sceneId);	
	res = sqlite3_exec(ctx->pDb, sqlCmd, NULL, NULL, &zErrMsg);
	if( res != SQLITE_OK ){
	   ERROR_PRINT("SQL error: %s\n", zErrMsg);
	   sqlite3_free(zErrMsg);
	   sqlite3_free(sqlCmd);
	   _kk_scene_unlock();	
	   return FAIL_RETURN;		
	}
	sqlite3_free(sqlCmd);
	_kk_scene_unlock();
	return SUCCESS_RETURN;
}


int kk_scene_parse_scene_trigger(const cJSON* str,const char *sceneId)
{
	int res = 0;
	if(str == NULL || sceneId == NULL){
		return INVALID_PARAMETER;
	}

	cJSON *triggers = cJSON_GetObjectItem(str,MSG_SCENE_TRIGGERS);
	if(triggers == NULL) return FAIL_RETURN;	
	cJSON *items = cJSON_GetObjectItem(triggers,MSG_SCENE_ITEMS);
	if(items == NULL) return FAIL_RETURN;
	cJSON * item = items->child;
	while(item != NULL){
		cJSON *type = cJSON_GetObjectItem(item,MSG_SCENE_TYPE);
		if(type == NULL) return FAIL_RETURN;
		cJSON *deviceCode = cJSON_GetObjectItem(item,MSG_DEVICE_CODE_STR);
		if(deviceCode == NULL) return FAIL_RETURN;	
		cJSON *epNum = cJSON_GetObjectItem(item,MSG_SCENE_EPNUM);
		if(epNum == NULL) return FAIL_RETURN;		
		cJSON *propertyName = cJSON_GetObjectItem(item,MSG_SCENE_PROPERTYNAME);
		if(propertyName == NULL) return FAIL_RETURN;
		cJSON *compareType = cJSON_GetObjectItem(item,MSG_SCENE_COMPARETYPE);
		if(compareType == NULL) return FAIL_RETURN;		
		cJSON *compareValue = cJSON_GetObjectItem(item,MSG_SCENE_COMPAREVALUE);
		if(compareValue == NULL) return FAIL_RETURN;	
		res = kk_scene_insert_scene_trigger(type->valuestring,deviceCode->valuestring,deviceCode->valueint,
				propertyName->valuestring,compareType->valuestring,compareValue->valuestring,sceneId);
		if(res != SUCCESS_RETURN){
			ERROR_PRINT("kk_scene_parse_scene_trigger fail!!!\n");
			return res;
		}
		item = item->next;	
	}
	INFO_PRINT("kk_scene_parse_scene_trigger success!!!\n");
	return SUCCESS_RETURN;
}
static int kk_scene_parse_repeatday(cJSON *repeatday)
{
	/***weekflag 01111111  ---> week 7 6 5 4 3 2 1*/
	char weekflag = 0;
	int iCnt = 0;
	if(repeatday == NULL){
		return INVALID_PARAMETER;
	}
	int  array_size = cJSON_GetArraySize (repeatday);
	INFO_PRINT("array_size:%d\n",array_size);
	if(array_size == 0){
		weekflag = 0;
	}else{
		for(iCnt = 0; iCnt < array_size; iCnt++){
			cJSON * pSub = cJSON_GetArrayItem(repeatday, iCnt);
			if(pSub != NULL){
				weekflag = weekflag|(1<<(pSub->valueint-1));
			}
		}
	}
	INFO_PRINT("kk_scene_parse_repeatday weekflag:%d!!!\n",weekflag);
	return weekflag;
	
}


int kk_scene_parse_scene_condition(const cJSON* str,const char *sceneId)
{
	char weekflag = 0;
	int res = 0;
	if(str == NULL || sceneId == NULL){
		ERROR_PRINT("kk_scene_parse_scene_condition failed\n");
		return INVALID_PARAMETER;
	}

	cJSON *conditon = cJSON_GetObjectItem(str,MSG_SCENE_CONDITIONS);
	if(conditon == NULL) return FAIL_RETURN;	

	cJSON *items = cJSON_GetObjectItem(conditon,MSG_SCENE_ITEMS);
	if(items == NULL) return FAIL_RETURN;
	cJSON * item = items->child;
	while(item != NULL){
		cJSON *type = cJSON_GetObjectItem(item,MSG_SCENE_TYPE);
		if(type == NULL) return FAIL_RETURN;

		cJSON *startTime = cJSON_GetObjectItem(item,MSG_SCENE_STARTTIME);
		if(startTime == NULL) return FAIL_RETURN;
		cJSON *endTime = cJSON_GetObjectItem(item,MSG_SCENE_ENDTIME);
		if(endTime == NULL) return FAIL_RETURN;	
		cJSON *crossday = cJSON_GetObjectItem(item,MSG_SCENE_CROSSDAY);
		if(crossday == NULL) return FAIL_RETURN;	
		cJSON *repeatday = cJSON_GetObjectItem(item,MSG_SCENE_REPEATDAYS);
		if(repeatday == NULL) return FAIL_RETURN;
		weekflag = kk_scene_parse_repeatday(repeatday);
		res = kk_scene_insert_scene_condition(type->valuestring,startTime->valueint,endTime->valueint,
				crossday->valueint,weekflag,sceneId);
		if(res != SUCCESS_RETURN){
			INFO_PRINT("kk_scene_insert_scene_condition fail!!!\n");
			return res;
		}
		item = item->next;
	}
	INFO_PRINT("kk_scene_parse_scene_condition success!!!\n");
	return SUCCESS_RETURN;
}

int kk_scene_parse_scene_action(const cJSON* str,const char *sceneId)
{
	int res = 0;
	if(str == NULL || sceneId == NULL){
		ERROR_PRINT("kk_scene_parse_scene_action failed\n");
		return INVALID_PARAMETER;
	}

	cJSON *action = cJSON_GetObjectItem(str,MSG_SCENE_ACTIONS);
	if(action == NULL) return FAIL_RETURN;
	
	//cJSON *items = cJSON_GetObjectItem(conditon,MSG_SCENE_ITEMS);
	//if(items == NULL) return FAIL_RETURN;
	cJSON * item = action->child;
	while(item != NULL){
		cJSON *type = cJSON_GetObjectItem(item,MSG_SCENE_TYPE);
		if(type == NULL) return FAIL_RETURN;	
		cJSON *deviceCode = cJSON_GetObjectItem(item,MSG_DEVICE_CODE_STR);
		if(deviceCode == NULL) return FAIL_RETURN;	
		cJSON *epNum = cJSON_GetObjectItem(item,MSG_SCENE_EPNUM);
		if(epNum == NULL) return FAIL_RETURN;	
		cJSON *propertyName = cJSON_GetObjectItem(item,MSG_SCENE_PROPERTYNAME);
		if(propertyName == NULL) return FAIL_RETURN;
		cJSON *propertyValue = cJSON_GetObjectItem(item,MSG_SCENE_PROPERTYVALUE);
		if(propertyValue == NULL) return FAIL_RETURN;
		cJSON *delay = cJSON_GetObjectItem(item,MSG_SCENE_DELAY);
		if(delay == NULL) return FAIL_RETURN;
		
		res = kk_scene_insert_scene_action(type->valuestring,deviceCode->valuestring,epNum->valueint,
				propertyName->valuestring,propertyValue->valuestring,delay->valueint,sceneId);
		if(res != SUCCESS_RETURN){
			INFO_PRINT("kk_scene_insert_scene_action fail!!!\n");
			return res;
		}		
		item = item->next;
	}
	INFO_PRINT("kk_scene_parse_scene_action success!!!\n");
	return SUCCESS_RETURN;
}


int kk_scene_parse_addscene(const cJSON* args,char *sceneId,int isUpdate)
{
	int res = 0;

	if(args == NULL||sceneId == NULL){
		ERROR_PRINT("[%d]kk_scene_parse_addscene fail!!!\n",__LINE__);
		return INVALID_PARAMETER;
	}
	
	cJSON *name = cJSON_GetObjectItem(args,MSG_SCENE_NAME);
	if(name == NULL) return FAIL_RETURN;
	cJSON *sceneType = cJSON_GetObjectItem(args,MSG_SCENE_SCENCTYPE);
	if(sceneType == NULL) return FAIL_RETURN;
	cJSON *enable = cJSON_GetObjectItem(args,MSG_SCENE_ENABLE);
	if(enable == NULL) return FAIL_RETURN;	
	if(!isUpdate){
		HAL_GetTimeMs(sceneId);//use time to create the sceneId
	}
	//sprintf(sceneId,"%lld",u64SceneKeyId);
	
	res = kk_scene_insert_scene_info(name->valuestring,sceneType->valueint,enable->valueint,sceneId);
	if(res != SUCCESS_RETURN){
		INFO_PRINT("kk_scene_insert_scene_info fail!!!\n");
		return res;
	}	

	res = kk_scene_parse_scene_trigger(args,sceneId);
	if(res != SUCCESS_RETURN){
		ERROR_PRINT("kk_scene_parse_scene_trigger failed\n");
		return FAIL_RETURN;
	}
	res = kk_scene_parse_scene_condition(args,sceneId);
	if(res != SUCCESS_RETURN){
		ERROR_PRINT("kk_scene_parse_scene_condition failed\n");
		return FAIL_RETURN;
	}	

	res = kk_scene_parse_scene_action(args,sceneId);
	if(res != SUCCESS_RETURN){
		ERROR_PRINT("kk_scene_parse_scene_action failed\n");
		return FAIL_RETURN;
	}		
	return SUCCESS_RETURN;

}
static void kk_scenec_delete(char* sceneId)
{
	kk_scene_delete_scene_info(sceneId);
	kk_scene_delete_scene_trigger(sceneId);
	kk_scene_delete_scene_condition(sceneId);	
	kk_scene_delete_scene_action(sceneId);
}
int kk_scene_parse_updatescene(const cJSON* arg,char *sceneId)
{

	if(arg == NULL||sceneId == NULL){
		ERROR_PRINT("[%d]kk_scene_parse_addscene fail!!!\n",__LINE__);
		return INVALID_PARAMETER;
	}
	kk_scenec_delete(sceneId);
	return kk_scene_parse_addscene(arg,sceneId,1);
}
int kk_scene_parse_deletescene(char *sceneId)
{

	if(sceneId == NULL){
		ERROR_PRINT("[%d]kk_scene_parse_deletescene fail!!!\n",__LINE__);
		return INVALID_PARAMETER;
	}
	kk_scenec_delete(sceneId);	
	return SUCCESS_RETURN;
}


